.rxjs-submit {
    margin-top: 30px;
    float: right !important;
}

.rjsf > .border-danger {
    display: none !important;
}

.progress {
    height: 20px !important;
}

.progress-bar {
    height: 20px !important;
    font-size: 16px !important;
}
